==> Bubot_AuthService-0.0.5.tar.gz: Bubot_AuthService-0.0.5/src/BubotObj/User/extension/AuthService/UserApi.py <==
import hashlib
import os
from base64 import b64encode, b64decode

from aiohttp import web
from bson.json_util import dumps

from Bubot.Helpers.ActionDecorator import async_action
from Bubot.Helpers.ExtException import KeyNotFound, Unauthorized
from BubotObj.Session.Session import Session
from BubotObj.User.UserApi import UserApi as BaseUserApi
from BubotObj.User.extension.AuthService.User import User


class UserApi(BaseUserApi):
    name = "User"
    handler = User
    file = __file__
    extension = True

    @async_action
    async def public_api_sign_in_by_password(self, view, **kwargs):
        action = kwargs['_action']
        try:
            login = view.data['login']
            password = view.data['password']
        except KeyError as err:
            raise KeyNotFound(detail=err)
        user = User(view.storage, lang=view.lang, form='CurrentUser')
        _auth = action.add_stat(await user.find_user_by_auth('password', login))
        bad_password = Unauthorized()

        _password = b64decode(_auth['password'])
        salt = _password[:32]
        if _auth['id'] != login or _password != self._generate_password_hash(salt, password):
            raise bad_password
        # _session = kwargs['session']
        session = action.add_stat(await Session.create_from_request(user, view))
        action.set_end(self.response.json_response({
            'session': str(session.obj_id),
            'user': user.data
        }))
        return action

    @async_action
    async def public_api_sign_up_by_password(self, view, **kwargs):
        action = kwargs['_action']
        try:
            login = view.data['login']
            password = view.data['password']
        except KeyError as err:
            raise KeyNotFound(detail=err)

        salt = os.urandom(32)
        password = b64encode(self._generate_password_hash(salt, password)).decode()

        user = User(view.storage, lang=view.lang, form='CurrentUser')
        res = action.add_stat(await user.add_auth({
            'type': 'password',
            'id': login,
            'password': password
        }, **kwargs))
        # if res:
        # if str(b64encode(password)) == res.get('password'):
        session = action.add_stat(await Session.create_from_request(user, view))
        return self.response.json_response({'session': str(session.obj_id)})

    @staticmethod
    def _generate_password_hash(salt, password):
        return salt + hashlib.pbkdf2_hmac('sha256', password.encode('utf-8'), salt, 100000)

    @async_action
    async def public_api_read_session_info(self, view, **kwargs):
        action = kwargs['_action']
        try:
            user_link = view.session.get('user')
            if not user_link:
                return web.json_response({"session": view.session.identity})
            # user = action.add_stat(self.handler.init_by_ref(user_link, lang=view.lang, form='CurrentUser'))
            user = User(view.storage, lang=view.lang, form='CurrentUser')
            action.add_stat(await user.find_by_id(user_link['_id'], _form=None))
            result = {
                "session": view.session.identity,
                "user": view.session.get('user'),
                "account": view.session.get('account'),
                "accounts": user.data.get('account', [])
            }
            return action.set_end(self.response.json_response(result, dumps=dumps))
            # login = view.data['login']
            # password = view.data['password']
        except KeyError as err:
            raise KeyNotFound(detail=err)

    @async_action
    async def api_sign_out(self, view, **kwargs):
        action = kwargs['_action']
        try:
            session = action.add_stat(await Session.init_from_request(view))
            await session.close()
            view.session.invalidate()
        except KeyError:  # нет сессии
            pass
        # session = await new_session(view.request)
        # session['last_visit'] = time.time()
        return action.set_end(self.response.json_response({}))

    # def _get_session_response(self, session, user):

==> Bubot_AuthService-0.0.6.tar.gz: Bubot_AuthService-0.0.6/src/BubotObj/User/extension/AuthService/UserApi.py <==
import hashlib
import os
from base64 import b64encode, b64decode
from datetime import datetime, timedelta

from aiohttp import web
from bson.json_util import dumps

from Bubot.Helpers.ActionDecorator import async_action
from Bubot.Helpers.ExtException import KeyNotFound, Unauthorized, AccessDenied
from BubotObj.Session.Session import Session
from BubotObj.User.UserApi import UserApi as BaseUserApi
from BubotObj.User.extension.AuthService.User import User


class UserApi(BaseUserApi):
    name = "User"
    handler = User
    file = __file__
    extension = True

    @async_action
    async def public_api_sign_in_by_password(self, view, *, _action=None, **kwargs):
        current_datetime = datetime.now()
        try:
            login = view.data['login']
            password = view.data['password']
        except KeyError as err:
            raise KeyNotFound(detail=err)
        user = User(view.storage, lang=view.lang, form='CurrentUser')
        _auth = _action.add_stat(await user.find_user_by_auth('password', login))
        _auth['bad_attempts'] = _auth.get('bad_attempts', 0)
        if _auth['bad_attempts']:
            next_attempt: datetime = _auth.get('next_attempt')
            if next_attempt and current_datetime < next_attempt:
                time_to_text_attempt = str(next_attempt - current_datetime).split('.')
                raise AccessDenied(message='Bad last login or password', detail=f"{time_to_text_attempt[0]}")
        bad_password = Unauthorized(message='Bad login or password')
        _password = b64decode(_auth['password'])
        salt = _password[:32]
        if _auth['id'] != login or _password != self._generate_password_hash(salt, password):
            _auth['bad_attempts'] += 1
            _auth['next_attempt'] = current_datetime + timedelta(seconds=30 * (_auth['bad_attempts'] - 1))
            _action.add_stat(await user.update_auth(_auth))
            raise bad_password
        # _session = kwargs['session']
        _auth['bad_attempts'] = 0
        _auth['next_attempt'] = current_datetime
        _action.add_stat(await user.update_auth(_auth))
        session = _action.add_stat(await Session.create_from_request(user, view))
        _action.set_end(self.response.json_response({
            'session': str(session.obj_id),
            'user': user.data
        }))
        return _action

    @async_action
    async def public_api_sign_up_by_password(self, view, **kwargs):
        action = kwargs['_action']
        try:
            login = view.data['login']
            password = view.data['password']
        except KeyError as err:
            raise KeyNotFound(detail=err)

        salt = os.urandom(32)
        password = b64encode(self._generate_password_hash(salt, password)).decode()

        user = User(view.storage, lang=view.lang, form='CurrentUser')
        res = action.add_stat(await user.add_auth({
            'type': 'password',
            'id': login,
            'password': password
        }, **kwargs))
        # if res:
        # if str(b64encode(password)) == res.get('password'):
        session = action.add_stat(await Session.create_from_request(user, view))
        return self.response.json_response({'session': str(session.obj_id)})

    @staticmethod
    def _generate_password_hash(salt, password):
        return salt + hashlib.pbkdf2_hmac('sha256', password.encode('utf-8'), salt, 100000)

    @async_action
    async def public_api_read_session_info(self, view, **kwargs):
        action = kwargs['_action']
        try:
            user_link = view.session.get('user')
            if not user_link:
                return web.json_response({"session": view.session.identity})
            # user = action.add_stat(self.handler.init_by_ref(user_link, lang=view.lang, form='CurrentUser'))
            user = User(view.storage, lang=view.lang, form='CurrentUser')
            action.add_stat(await user.find_by_id(user_link['_id'], _form=None))
            result = {
                "session": view.session.identity,
                "user": view.session.get('user'),
                "account": view.session.get('account'),
                "accounts": user.data.get('account', [])
            }
            return action.set_end(self.response.json_response(result, dumps=dumps))
            # login = view.data['login']
            # password = view.data['password']
        except KeyError as err:
            raise KeyNotFound(detail=err)

    @async_action
    async def api_sign_out(self, view, **kwargs):
        action = kwargs['_action']
        try:
            session = action.add_stat(await Session.init_from_request(view))
            await session.close()
            view.session.invalidate()
        except KeyError:  # нет сессии
            pass
        # session = await new_session(view.request)
        # session['last_visit'] = time.time()
        return action.set_end(self.response.json_response({}))

    # def _get_session_response(self, session, user):
